/* This file is Copyright 2002 Level Control Systems.  See the included LICENSE.txt file for details. */

#include <new>
#include <typeinfo>
#include <string.h>

#include "system/GlobalMemoryAllocator.h"

namespace muscle {

static MemoryAllocatorRef _globalAllocatorRef;

void SetCPlusPlusGlobalMemoryAllocator(MemoryAllocatorRef maRef)
{
   MemoryAllocatorRef holdPrev = _globalAllocatorRef;  // necessary to ensure correct order of operations!
   if (_globalAllocatorRef != maRef)
   {
      // Gotta do some special juju here so that _globalAllocatorRef's destructor
      // doesn't cause the referenced object to be deleted, because then it
      // will try to delete itself and crash.  So better just to leave it
      // allocated and let the system clean it up.
      if (_globalAllocatorRef()) _globalAllocatorRef()->DecrementRefCount();
      _globalAllocatorRef = maRef;
      if (_globalAllocatorRef()) _globalAllocatorRef()->IncrementRefCount();
   }
}

MemoryAllocatorRef GetCPlusPlusGlobalMemoryAllocator()
{
   return _globalAllocatorRef;
}

};  // end namespace muscle

// Metrowerk's compiler crashes at run-time if this memory-tracking
// code is enabled.  I haven't been able to figure out why (everything
// looks okay, but after a few dozen malloc()/free() calls, free()
// crashes!) so I'm just going to disable this code for PowerPC/Metrowerks
// machines.  Sorry!   --jaf 12/01/00
#ifdef __MWERKS__
# define DISABLE_MUSCLE_MEMORY_TRACKING
#endif

#ifndef DISABLE_MUSCLE_MEMORY_TRACKING

// VC++ doesn't know about these  :^(
#ifdef WIN32
# define bad_alloc()
# define nothrow_t
#endif

#ifdef WIN32
void * operator new(size_t s)
#else
void * operator new(size_t s) throw (bad_alloc)
#endif
{
   using namespace muscle;
   void * ret = (_globalAllocatorRef()) ? _globalAllocatorRef()->Malloc(s) : malloc(s);
   if (ret == NULL) throw bad_alloc();
   return ret;
}

#ifdef WIN32
void * operator new[](size_t s)
#else
void * operator new[](size_t s) throw (bad_alloc)
#endif
{
   using namespace muscle;
   void * ret = (_globalAllocatorRef()) ? _globalAllocatorRef()->Malloc(s) : malloc(s);
   if (ret == NULL) throw bad_alloc();
   return ret;
}

#ifdef WIN32
void * operator new(size_t s, nothrow_t const &)
#else
void * operator new(size_t s, nothrow_t const &) throw ()
#endif
{
   using namespace muscle;
   return (_globalAllocatorRef()) ? _globalAllocatorRef()->Malloc(s) : malloc(s);
}

#ifdef WIN32
void * operator new[](size_t s, nothrow_t const &)
#else
void * operator new[](size_t s, nothrow_t const &) throw ()
#endif
{
   using namespace muscle;
   return (_globalAllocatorRef()) ? _globalAllocatorRef()->Malloc(s) : malloc(s);
}

#ifdef WIN32
void operator delete(void * p)
#else
void operator delete(void * p) throw()
#endif
{
   using namespace muscle;
   if (p) 
   {
      if (_globalAllocatorRef()) _globalAllocatorRef()->Free(p);
                       else free(p);
   }
}

#ifdef WIN32
void operator delete[](void * p)
#else
void operator delete[](void * p) throw()
#endif
{
   using namespace muscle;
   if (p) 
   {
      if (_globalAllocatorRef()) _globalAllocatorRef()->Free(p);
                       else free(p);
   }
}

#endif
