/* This file is Copyright 2002 Level Control Systems.  See the included LICENSE.txt file for details. */ 

#include "util/MemoryAllocator.h"

namespace muscle {

MemoryAllocator :: MemoryAllocator()
{
   // empty
}

MemoryAllocator :: ~MemoryAllocator()
{
   // empty
}

BasicMemoryAllocator :: BasicMemoryAllocator() : _hasMallocFailed(false)
{
   // empty
}

BasicMemoryAllocator :: ~BasicMemoryAllocator()
{
   // empty
}

void * BasicMemoryAllocator :: Malloc(uint32 howMuch)
{
   void * ret = malloc(howMuch);
   if (ret == NULL) _hasMallocFailed = true;
   return ret;
}

void BasicMemoryAllocator :: Free(void * memory)
{
   free(memory);
}

bool BasicMemoryAllocator :: HasMallocFailed(bool setFlagTo) const
{
   bool ret = _hasMallocFailed;
   _hasMallocFailed = setFlagTo;
   return ret;
}

UsageLimitProxyMemoryAllocator :: UsageLimitProxyMemoryAllocator(MemoryAllocatorRef slaveRef, uint32 maxBytes, uint32 currentBytes) : ProxyMemoryAllocator(slaveRef), _maxBytes(maxBytes), _currentBytes(currentBytes)
{
   // empty
}
 
UsageLimitProxyMemoryAllocator :: ~UsageLimitProxyMemoryAllocator()
{
   // empty
}
 
void * UsageLimitProxyMemoryAllocator :: Malloc(uint32 howMuch)
{
   if (_currentBytes + howMuch > _maxBytes) 
   {
      (void) ProxyMemoryAllocator::HasMallocFailed(true);  // force an error condition
      return NULL;
   }
   uint32 allocSize = howMuch + sizeof(uint32);  // requested size, plus 4 bytes for our use
   uint32 * a = (uint32 *)(ProxyMemoryAllocator::Malloc(allocSize));
   if (a)
   {
      *a = allocSize;  // our little header tag so that Free() will know how big the allocation was
      _currentBytes += allocSize;
//printf("+%li = %lu\n", allocSize, _currentBytes);
      return a+1;  // user doesn't want to see the header tag, of course
   }
   else return NULL;
}

void UsageLimitProxyMemoryAllocator :: Free(void * memory)
{
   uint32 * realStart = ((uint32*)memory)-1;
   uint32 s = *realStart;
   _currentBytes -= s;
//printf("-%li = %lu\n", s, _currentBytes);
   ProxyMemoryAllocator::Free(realStart);
}

AutoCleanupProxyMemoryAllocator :: AutoCleanupProxyMemoryAllocator(MemoryAllocatorRef slaveRef) : ProxyMemoryAllocator(slaveRef)
{
   // empty
}

AutoCleanupProxyMemoryAllocator :: ~AutoCleanupProxyMemoryAllocator()
{
   ClearCallbacks();
}

status_t AutoCleanupProxyMemoryAllocator :: AddCallback(OutOfMemoryCallbackRef cbRef)
{
   return _callbacks.AddTail(cbRef);
}

void AutoCleanupProxyMemoryAllocator :: ClearCallbacks()
{
   _callbacks.Clear();
}

void * AutoCleanupProxyMemoryAllocator :: Malloc(uint32 howMuch)
{
   void * ret = ProxyMemoryAllocator::Malloc(howMuch);
   if ((ret == NULL)&&(_callbacks.GetNumItems() > 0))
   {
      // oops!  Let's see if our callbacks can free up some mem, then we'll try again
      uint32 nc = _callbacks.GetNumItems();
      for (uint32 i=0; i<nc; i++) if (_callbacks[i]()) (_callbacks[i]())->OutOfMemory();
      (void) ProxyMemoryAllocator::HasMallocFailed(false);  // force-clear the error flag
      ret = ProxyMemoryAllocator::Malloc(howMuch);  // and try one more time
   }
   return ret;
}

OutOfMemoryCallback :: OutOfMemoryCallback()
{
   // empty
}

OutOfMemoryCallback :: ~OutOfMemoryCallback()
{
   // empty
}

};  // end namespace muscle
